package ch.zhaw.projekt2.turnierverwaltung;

import ch.zhaw.projekt2.turnierverwaltung.main.gameScheduleView.GameController;
import ch.zhaw.projekt2.turnierverwaltung.main.gameScheduleView.GameDecorator;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class FactoryDecorator implements IsObservable{
    private Factory factory;
    private FileIO fileIO;
    private Pane pane;
    private List<IsObserver> listener = new ArrayList<>();

    public FactoryDecorator(FileIO fileIO, Factory factory, Pane pane){
        setFactory(factory);
        setFileIO(fileIO);
        setPane(pane);
    }

    public void setFileIO(FileIO fileIO) {
        this.fileIO = fileIO;
    }

    public void setFactory(Factory factory) {
        this.factory = factory;
    }

    public void setPane(Pane pane) {
        this.pane = pane;
    }

    @Override
    public void addListener(IsObserver observer) {
        listener.add(observer);
    }

    @Override
    public void removeListener(IsObserver observer) {
        listener.remove(observer);
    }

    public void openTournament(FileIO.TournamentFile tournamentFile){
        try {
            factory.setTournament(fileIO.loadTournament(tournamentFile));
            openScheduleView();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } //TODO handle and logging
    }

    public void openTournamentList() {
        factory.loadTournamentList((BorderPane) pane, this);
        informListener();
    }

    public void openParticipantFormular() {
        factory.loadParticipantFormular((BorderPane) pane, this);
        informListener();
    }

    public void openPlacesFormular() {
        factory.loadPlacesFormular((BorderPane) pane, this);
        informListener();
    }

    public void openScheduleView() {
        factory.loadGameScheduler((BorderPane) pane, this);
        informListener();
    }

    public void loadGameTree(HBox hBoxCenter, TournamentDecorator tournamentDecorator) {
        hBoxCenter.getChildren().clear();

        List<List<Game>> gameList = tournamentDecorator.getTournament().getGameList();

        List<GameDecorator> gameDecorators = new ArrayList<>();
        for (int i = 0; i < gameList.size(); i++) {
            List<GameDecorator> newGameDecorators = new ArrayList<>();
            VBox vBox = new VBox();
            for (int j = 0; j < gameList.get(i).size(); j++) {
                GameDecorator gameDecorator = new GameDecorator(gameList.get(i).get(j));
                newGameDecorators.add(gameDecorator);
                GameController controller = openGameView(vBox,gameDecorator);
                if(i>0){
                    controller.addListener(gameDecorators.get(j*2));
                    controller.addListener(gameDecorators.get(j*2+1));
                }
                gameDecorator.addListener(new IsObserver() {
                    @Override
                    public void update() {
                        tournamentDecorator.saveTournament();
                    }
                });
                controller.loadContent();
            }
            hBoxCenter.getChildren().add(vBox);
            gameDecorators = newGameDecorators;
        }
    }

    public GameController openGameView(VBox vBox, GameDecorator gameDecorator) {
        return factory.loadGameView(vBox ,gameDecorator, this);
    }


    public void informListener() {
        for(IsObserver observer : listener) {
            observer.update();
        }
    }
}
